@@ -1,4 +1,4 @@
-/* Copyright (c) 1998, 1999, 2001, 2002 Thorsten Kukuk
+/* Copyright (c) 1998, 1999, 2001, 2002, 2004, 2005 Thorsten Kukuk
    This file is part of ypbind-mt.
    Author: Thorsten Kukuk <kukuk@suse.de>
 
@@ -168,35 +168,66 @@ load_config (int check_syntax)
 	  /* We have
 	     domain <domain> server <host|ip>
 	     or
-	     domain <domain> broadcast*/
+	     domain <domain> broadcast
+	     or
+	     domain<domain> slp
+	  */
 
-	  count = sscanf (cp, "domain %64s server %80s", tmpdomain,
-			  tmpserver);
-	  if (count == 2)
+	  if (strstr (cp, "server") != NULL)
 	    {
-	      if (debug_flag)
-		log_msg (LOG_DEBUG, _("parsed domain '%s' server '%s'"),
-			 tmpdomain, tmpserver);
-	      if (add_server (tmpdomain, tmpserver, check_syntax))
-		++have_entries;
-	      else
-		++bad_entries;
+	      count = sscanf (cp, "domain %64s server %80s", tmpdomain,
+			      tmpserver);
+	      if (count == 2)
+		{
+		  if (debug_flag)
+		    log_msg (LOG_DEBUG, _("parsed domain '%s' server '%s'"),
+			     tmpdomain, tmpserver);
+		  if (add_server (tmpdomain, tmpserver, check_syntax))
+		    ++have_entries;
+		  else
+		    ++bad_entries;
 
-	      continue;
+		  continue;
+		}
 	    }
-	  count = sscanf (cp, "domain %s broadcast", tmpdomain);
-	  if (count == 1)
+	  if (strstr (cp, "broadcast") != NULL)
 	    {
-	      if (debug_flag)
-		log_msg (LOG_DEBUG, _("parsed domain '%s' broadcast"),
-			 tmpdomain);
-	      if (add_server (tmpdomain, NULL, check_syntax))
-		++have_entries;
-	      else
-		++bad_entries;
+	      count = sscanf (cp, "domain %s broadcast", tmpdomain);
+	      if (count == 1)
+		{
+		  if (debug_flag)
+		    log_msg (LOG_DEBUG, _("parsed domain '%s' broadcast"),
+			     tmpdomain);
+		  if (add_server (tmpdomain, NULL, check_syntax))
+		    ++have_entries;
+		  else
+		    ++bad_entries;
 
-	      continue;
+		  continue;
+		}
 	    }
+#ifdef USE_SLP
+	  if (strstr (cp, "slp") != NULL)
+	    {
+	      count = sscanf (cp, "domain %s slp", tmpdomain);
+	      if (count == 1)
+		{
+		  int i;
+
+		  if (debug_flag)
+		    log_msg (LOG_DEBUG, _("parsed domain '%s' slp"),
+			     tmpdomain);
+		  i = query_slp (tmpdomain);
+
+		  if (i > 0)
+		    have_entries += 1;
+		  else
+		    ++bad_entries;
+
+		  continue;
+		}
+	    }
+#endif
 	}
       else if (strncmp (cp, "ypserver", 8) == 0 && isspace ((int)cp[8]))
 	{
@@ -378,7 +409,13 @@ sig_handler (void *v_param  __attribute__ ((unused)))
 
   while (1)
     {
-      sigwait (&sigs_to_catch, &caught);
+      int ret = sigwait (&sigs_to_catch, &caught);
+      if (ret != 0)
+	{
+	  if (ret != EINTR)
+	    log_msg (LOG_ERR, _("sigwait failed: ret=%d."), ret);
+	  continue;
+	}
       switch (caught)
 	{
 	case SIGCHLD:
@@ -431,7 +468,7 @@ static void
 usage (void)
 {
   fputs (_("Usage:\n"), stderr);
-  fputs (_("\typbind [-broadcast | -ypset | -ypsetme] [-p port] [-f configfile]\n\t  [-no-ping] [-broken-server] [-local_only] [-debug]\n"), stderr);
+  fputs (_("\typbind [-broadcast | -ypset | -ypsetme] [-p port] [-f configfile]\n\t  [-no-ping] [-broken-server] [-local-only] [-i ping-interval] [-debug]\n"), stderr);
   fputs (_("\typbind -c [-f configfile]\n"), stderr);
   fputs (_("\typbind --version\n"), stderr);
   exit (1);
@@ -476,8 +513,8 @@ main (int argc, char **argv)
 	ping_interval = 0;
       else if (strcmp ("-broadcast", argv[i]) == 0)
 	use_broadcast = 1;
-      else if (strcmp ("-local_only", argv[i]) == 0 ||
-	       strcmp ("-local-only", argv[i]) == 0)
+      else if (strcmp ("-local-only", argv[i]) == 0 ||
+	       strcmp ("-local_only", argv[i]) == 0)
 	local_only = 1;
       else if (strcmp ("-f", argv[i]) == 0)
 	{
@@ -493,6 +530,15 @@ main (int argc, char **argv)
 	  ++i;
 	  port = atoi (argv[i]);
 	}
+      else if (strcmp ("-ping-interval", argv[i]) == 0 ||
+	       strcmp ("-ping-interval", argv[i]) == 0 ||
+	       strcmp ("-i", argv[i]) == 0)
+	{
+	  if (i+1 == argc || argv[i+1][0] == '-')
+	    usage ();
+	  ++i;
+	  ping_interval = atoi (argv[i]);
+	}
       else if (strcmp ("-c", argv[i]) == 0)
 	configcheck_only = 1;
       else
@@ -579,7 +625,10 @@ main (int argc, char **argv)
   openlog ("ypbind", LOG_PID, LOG_DAEMON);
 
   if (debug_flag)
-    log_msg (LOG_DEBUG, "[Welcome to ypbind-mt, version %s]\n", VERSION);
+    {
+      log_msg (LOG_DEBUG, "[Welcome to ypbind-mt, version %s]\n", VERSION);
+      log_msg (LOG_DEBUG, "ping interval is %d seconds\n", ping_interval);
+    }
   else
     {
       int j;
@@ -663,10 +712,10 @@ main (int argc, char **argv)
       if (local_only)
 	socket_address.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
       else
-	{
-	  socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
-	  socket_address.sin_port = htons (port);
-	}
+	socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
+
+      if (port >= 0)
+	socket_address.sin_port = htons (port);
 
       result = bind (sock, (struct sockaddr *) &socket_address,
 		     sizeof (socket_address));
@@ -716,10 +765,10 @@ main (int argc, char **argv)
       if (local_only)
 	socket_address.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
       else
-	{
-	  socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
-	  socket_address.sin_port = htons (port);
-	}
+	socket_address.sin_addr.s_addr = htonl (INADDR_ANY);
+
+      if (port >= 0)
+	socket_address.sin_port = htons (port);
 
       result = bind (sock, (struct sockaddr *) &socket_address,
 		     sizeof (socket_address));
